@@ -1,6 +1,9 @@
 #------------------------------------------------------------------------------
-#  Copyright (c) 2011, Enthought, Inc.
-#  All rights reserved.
+# Copyright (c) 2013, Nucleic Development Team.
+#
+# Distributed under the terms of the Modified BSD License.
+#
+# The full license is in the file COPYING.txt, distributed with this software.
 #------------------------------------------------------------------------------
 import os
 import tokenize
@@ -39,18 +42,18 @@ def _parsing_error(klass, message, token):
     ----------
     klass : Exception class
         The exception type to raise.
-    
+
     message : string
         The message to pass to the exception.
-    
+
     token : LexToken
         The current lextoken for the error.
-    
+
     """
     filename = token.lexer.filename
     lexdata = token.lexer.lexdata
     lineno = token.lineno
-    text = lexdata.splitlines()[lineno - 1] 
+    text = lexdata.splitlines()[lineno - 1]
     info = (filename, lineno, 0, text)
     raise ParsingError(klass, message, info)
 
@@ -63,7 +66,7 @@ def syntax_error(message, token):
     ----------
     message : string
         The message to pass to the IndentationError
-    
+
     token : LexToken
         The current lextoken for the error.
 
@@ -71,6 +74,24 @@ def syntax_error(message, token):
     _parsing_error(SyntaxError, message, token)
 
 
+def syntax_warning(message, token):
+    """ Raise a warning about dubious syntax.
+
+    Parameters
+    ----------
+    message : string
+        The message to pass to the IndentationError
+
+    token : LexToken
+        The current lextoken for the error.
+
+    """
+    import warnings
+    filename = token.lexer.filename
+    lineno = token.lineno
+    warnings.warn_explicit(message, SyntaxWarning, filename, lineno)
+
+
 def indentation_error(message, token):
     """ Raise a ParsingError which will be converted into an proper
     IndentationError during parsing.
@@ -79,7 +100,7 @@ def indentation_error(message, token):
     ----------
     message : string
         The message to pass to the IndentationError
-    
+
     token : LexToken
         The current lextoken for the error.
 
@@ -132,6 +153,7 @@ class EnamlLexer(object):
         (r'::', 'DOUBLECOLON'),
         (r'\.\.\.', 'ELLIPSIS'),
         (r':=', 'COLONEQUAL'),
+        (r'=>', 'RIGHTARROW'),
     )
 
     tokens = (
@@ -148,6 +170,7 @@ class EnamlLexer(object):
         'RBRACE',
         'RPAR',
         'RSQB',
+        'PRAGMA',
         'STRING',
         'WS',
 
@@ -161,10 +184,12 @@ class EnamlLexer(object):
 
     reserved = {
         'and': 'AND',
+        'alias': 'ALIAS',
         'as': 'AS',
         'assert': 'ASSERT',
         'break': 'BREAK',
         'class': 'CLASS',
+        'const': 'CONST',
         'continue': 'CONTINUE',
         'def': 'DEF',
         'del': 'DEL',
@@ -188,14 +213,15 @@ class EnamlLexer(object):
         'print': 'PRINT',
         'raise': 'RAISE',
         'return': 'RETURN',
+        'template': 'TEMPLATE',
         'try': 'TRY',
         'while': 'WHILE',
         'with': 'WITH',
         'yield': 'YIELD',
     }
 
-    tokens = (tokens + 
-              tuple(val[1] for val in operators) + 
+    tokens = (tokens +
+              tuple(val[1] for val in operators) +
               tuple(reserved.values()))
 
     #--------------------------------------------------------------------------
@@ -213,16 +239,17 @@ class EnamlLexer(object):
     #--------------------------------------------------------------------------
     t_COMMA = r','
     t_NUMBER = tokenize.Number
+    t_PRAGMA = r'\$pragma'
 
     # Generate the token matching rules for the operators
     for tok_pattern, tok_name in operators:
         tok_name = 't_' + tok_name
         locals()[tok_name] = tok_pattern
-    
+
     def t_comment(self, t):
         r'[ ]*\#[^\r\n]*'
         pass
-   
+
     def t_WS(self, t):
         r' [ \t\f]+ '
         value = t.value
@@ -231,7 +258,7 @@ class EnamlLexer(object):
         # line; it will be ignored for the indentation calculations
         # above. Formfeed characters occurring elsewhere in the
         # leading whitespace have an undefined effect (for instance,
-        # they may reset the space count to zero). 
+        # they may reset the space count to zero).
         value = value.rsplit("\f", 1)[-1]
 
         # First, tabs are replaced (from left to right) by one to eight
@@ -267,7 +294,7 @@ class EnamlLexer(object):
         t.type = 'NEWLINE'
         if self.paren_count == 0:
             return t
-    
+
     def t_LPAR(self, t):
         r'\('
         self.paren_count += 1
@@ -277,7 +304,7 @@ class EnamlLexer(object):
         r'\)'
         self.paren_count -= 1
         return t
-    
+
     def t_LBRACE(self, t):
         r'\{'
         self.paren_count += 1
@@ -287,7 +314,7 @@ class EnamlLexer(object):
         r'\}'
         self.paren_count -= 1
         return t
-    
+
     def t_LSQB(self, t):
         r'\['
         self.paren_count += 1
@@ -318,7 +345,7 @@ class EnamlLexer(object):
             self.is_raw = True
         t.value = t.value.split("'", 1)[0]
         return t
-    
+
     def t_TRIPLEQ1_simple(self, t):
         r"[^'\\]+"
         t.type = "STRING_CONTINUE"
@@ -336,7 +363,7 @@ class EnamlLexer(object):
         t.lexer.pop_state()
         self.is_raw = False
         return t
-    
+
     # supress PLY warning
     t_TRIPLEQ1_ignore = ''
 
@@ -402,10 +429,10 @@ class EnamlLexer(object):
         t.lexer.pop_state()
         self.is_raw = False
         return t
-    
+
     # supress PLY warning
     t_SINGLEQ1_ignore = ''
-    
+
     def t_SINGLEQ1_error(self, t):
         syntax_error('EOL while scanning single quoted string.', t)
 
@@ -432,13 +459,13 @@ class EnamlLexer(object):
         t.lexer.pop_state()
         self.is_raw = False
         return t
-    
+
     # supress PLY warning
     t_SINGLEQ2_ignore = ''
-    
+
     def t_SINGLEQ2_error(self, t):
         syntax_error('EOL while scanning single quoted string.', t)
-    
+
     #--------------------------------------------------------------------------
     # Miscellaneous Token Rules
     #--------------------------------------------------------------------------
@@ -450,7 +477,7 @@ class EnamlLexer(object):
 
     def t_error(self, t):
         syntax_error('invalid syntax', t)
-        
+
     #--------------------------------------------------------------------------
     # Normal Class Items
     #--------------------------------------------------------------------------
@@ -461,11 +488,11 @@ class EnamlLexer(object):
         self.token_stream = None
         self.filename = filename
 
-        # Ply has a bit of an inconsistency when using a class as a 
-        # lexer instead of a module. The .lexer attribute of tokens 
-        # created by the ply lexer are set to that ply lexer instance, 
-        # but the .lexer attribute of tokens created by the ply parser 
-        # are set to the instance of this class. So, when the p_error 
+        # Ply has a bit of an inconsistency when using a class as a
+        # lexer instead of a module. The .lexer attribute of tokens
+        # created by the ply lexer are set to that ply lexer instance,
+        # but the .lexer attribute of tokens created by the ply parser
+        # are set to the instance of this class. So, when the p_error
         # function is called in the parser, we can't be sure which lexer
         # will be set on the error token. Since we need the filename in
         # that function, we add it as an attribute on both lexers.
@@ -486,7 +513,7 @@ class EnamlLexer(object):
             return tok
         except StopIteration:
             pass
-   
+
     def dedent(self, lineno):
         # Synthesize a DEDENT Token
         tok = lex.LexToken()
@@ -518,12 +545,23 @@ class EnamlLexer(object):
 
     def make_token_stream(self):
         token_stream = iter(self.lexer.token, None)
+        token_stream = self.ensure_token_lexer(token_stream)
         token_stream = self.create_strings(token_stream)
         token_stream = self.annotate_indentation_state(token_stream)
         token_stream = self.synthesize_indentation_tokens(token_stream)
         token_stream = self.add_endmarker(token_stream)
         return token_stream
 
+    def ensure_token_lexer(self, token_stream):
+        # PLY only assigns the lexer to tokens which are passed
+        # to t_* functions. This ensures each token gets assigned
+        # a lexer, since that is required by the error handlers.
+        lexer = self.lexer
+        for tok in token_stream:
+            if getattr(tok, 'lexer', None) is None:
+                tok.lexer = lexer
+            yield tok
+
     def create_strings(self, token_stream):
         for tok in token_stream:
             if not tok.type.startswith("STRING_START_"):
@@ -591,21 +629,21 @@ class EnamlLexer(object):
     #  0) no colon hence no need to indent
     #  1) "if 1: go()" - simple statements have a COLON but no need for an indent
     #  2) "if 1:\n  go()" - complex statements have a COLON NEWLINE and must indent
-    # 
+    #
     # We only care about whitespace at the start of a line
     def annotate_indentation_state(self, token_stream):
         NO_INDENT = 0
         MAY_INDENT = 1
         MUST_INDENT = 2
-        
+
         self.at_line_start = at_line_start = True
         indent = NO_INDENT
-        
+
         for token in token_stream:
             token.at_line_start = at_line_start
 
             # The double colon serves double purpose: in slice operations
-            # and also as the notification operator. In the case of a 
+            # and also as the notification operator. In the case of a
             # slice operation, newline continuation is already allowed
             # by suppressing NEWLINE tokens in a multiline expression.
             # So, we can treat double colon the same as colon here for
@@ -614,7 +652,7 @@ class EnamlLexer(object):
                 at_line_start = False
                 indent = MAY_INDENT
                 token.must_indent = False
-            
+
             elif token.type == "NEWLINE":
                 at_line_start = True
                 if indent == MAY_INDENT:
@@ -668,7 +706,7 @@ class EnamlLexer(object):
                 # pass the other cases on through
                 yield token
                 continue
-            
+
             # then it must be a real token (not WS, not NEWLINE)
             # which can affect the indentation level
             prev_was_ws = False
@@ -704,20 +742,20 @@ class EnamlLexer(object):
 
         # If the current token is WS (which is only emitted at the start
         # of a line), then the token before that was a newline unless
-        # we're on line number 1. If that's the case, then we don't 
+        # we're on line number 1. If that's the case, then we don't
         # need another newline token.
         if token is None:
-            yield self.newline(-1)
+            yield self.newline(self.lexer.lineno)
         elif token.type != 'NEWLINE':
             if token.type != 'WS' or token.lineno == 1:
-                yield self.newline(-1)
+                yield self.newline(self.lexer.lineno)
 
         # Must dedent any remaining levels
         if len(levels) > 1:
             assert token is not None
             for _ in range(1, len(levels)):
                 yield self.dedent(token.lineno)
-        
+
     def add_endmarker(self, token_stream):
         for tok in token_stream:
             yield tok
