@@ -274,7 +274,6 @@ class LayoutManager(Atom):
 
         """
         self._root_item = item
-        self.set_items([])
 
     def set_items(self, items):
         """ Set the layout items for this layout manager.
@@ -333,6 +332,12 @@ class LayoutManager(Atom):
         # Store the layout items for resize updates.
         self._layout_items = items
 
+    def clear_items(self):
+        """ Clear the child layout items in the layout.
+
+        """
+        del self._layout_items
+
     def resize(self, width, height):
         """ Update the size of target size of the layout.
 
