@@ -15,7 +15,7 @@ from enaml.widgets.constraints_widget import ConstraintsWidget
 from enaml.widgets.container import ProxyContainer
 
 from .QtCore import QRect, QSize, QTimer, Signal
-from .QtGui import QFrame, QWidgetItem
+from .QtGui import QFrame, QWidgetItem, QSizePolicy
 
 from .qt_constraints_widget import QtConstraintsWidget
 from .qt_frame import QtFrame
@@ -26,6 +26,10 @@ DEFAULT_BEST_SIZE = QSize(-1, -1)
 DEFAULT_MIN_SIZE = QSize(0, 0)
 DEFAULT_MAX_SIZE = QSize(16777215, 16777215)
 
+# The size policy to apply to containers. This allows the container
+# to behave properly when added to standard Qt layouts and widgets.
+CONTAINER_POLICY = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+
 
 class LayoutPoint(Atom):
     """ A class which represents a point in layout space.
@@ -102,7 +106,7 @@ class QtLayoutItem(LayoutItem):
             constraints will not be generated for that dimension.
 
         """
-        min_size = self.widget_item.minimumSize()
+        min_size = self.widget_item.widget().minimumSize()
         if min_size != DEFAULT_MIN_SIZE:
             return (min_size.width(), min_size.height())
         return (-1, -1)
@@ -118,7 +122,7 @@ class QtLayoutItem(LayoutItem):
             constraints will not be generated for that dimension.
 
         """
-        max_size = self.widget_item.maximumSize()
+        max_size = self.widget_item.widget().maximumSize()
         if max_size != DEFAULT_MAX_SIZE:
             return (max_size.width(), max_size.height())
         return (-1, -1)
@@ -318,7 +322,9 @@ class QtContainer(QtFrame, ProxyContainer):
         """ Creates the QContainer widget.
 
         """
-        self.widget = QContainer(self.parent_widget())
+        widget = QContainer(self.parent_widget())
+        widget.setSizePolicy(CONTAINER_POLICY)
+        self.widget = widget
 
     def init_layout(self):
         """ Initialize the layout of the widget.
@@ -352,14 +358,14 @@ class QtContainer(QtFrame, ProxyContainer):
 
         """
         # If this container owns the layout, (re)start the timer. The
-        # list of layout items is reset to prevent an edge case where
+        # list of layout items is cleared to prevent an edge case where
         # a parent container layout occurs before the child container,
         # causing the child to resize potentially deleted widgets which
         # still have strong refs in the layout items list.
         manager = self._layout_manager
         if manager is not None:
             if self._layout_timer is None:
-                manager.set_items([])
+                manager.clear_items()
                 self.widget.setUpdatesEnabled(False)
                 timer = self._layout_timer = QTimer()
                 timer.setSingleShot(True)
